@@ -36,16 +36,14 @@ def main(project):
 
     #Option parser
     description=textwrap.dedent('''\
-    This script queries an ESGF archive. It can query a
-    local POSIX-based archive following the CMIP5 DRS
+    This script queries a {0} archive. It can query:
+    1. a local POSIX-based archive that follows the {0} DRS
     filename convention and directory structure.
+    2. the ESGF {0} archive.
 
-    In the future it should become able to query the THREDDS
-    catalog of the ESGF and provide a simple interface to
-    the OPEnDAP services.
-    ''')
-    epilog='Frederic Laliberte, Paul Kushner 10/2012'
-    version_num='0.2'
+    '''.format(project))
+    epilog='Frederic Laliberte, Paul Kushner 02/2014'
+    version_num='0.9'
     parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                             description=description,
                             version='%(prog)s '+version_num,
@@ -65,13 +63,13 @@ def main(project):
 
     #Load pointer file:
     if options.command=='remote_retrieve':
-        netcdf_utils.retrieve_data(options)
+        netcdf_utils.retrieve_data(options,project_drs)
     elif 'in_diagnostic_headers_file' in dir(options):
         paths_dict=cdb_query_archive_class.SimpleTree(io_tools.open_json(options),options,project_drs)
         #Run the command:
         getattr(paths_dict,options.command)(options)
     elif 'in_diagnostic_netcdf_file' in dir(options):
-        paths_dict=cdb_query_archive_class.SimpleTree(io_tools.open_netcdf(options),options,project_drs)
+        paths_dict=cdb_query_archive_class.SimpleTree(io_tools.open_netcdf(options,project_drs),options,project_drs)
         #Run the command:
         getattr(paths_dict,options.command)(options)
         
