@@ -19,7 +19,7 @@ def open_netcdf(options,project_drs):
     infile=netCDF4.Dataset(options.in_diagnostic_netcdf_file,'r')
     paths_dict=dict()
     paths_dict['header']=dict()
-    for att in project_drs.header_desc:
+    for att in set(project_drs.header_desc).intersection(infile.ncattrs()):
         paths_dict['header'][att]=json.loads(infile.getncattr(att))
     infile.close()
     return paths_dict
